/* 	

		Fitting functions fit(), worstfit(), bestfit() of Arvert 4.x
		P. Zeitler, Lehigh University
		2/16/04
		
		fit() can use either mean percent deviation or mswd
		
		mean percent deviation is just the arithmetic average of the absolute value
		of the percent deviation between measured and goals steps, for each fitted step.
		
		2/16/04 -- routine changed to calculate mswd rather than chi-square as the second fitting option,
		and added bestfit() routine
  		4/04 -- added code for optional fitting of He ages
  	
*/

#include "arvert.h"

double fit(long index, double **poolAge)
{

	extern long goalsteps;
	extern double fitmax;
	extern double goal_loss[101], goal_age[101], goal_error[101];
	extern long fitchoice, goal_skip[101];
	extern double Heweight, Hepool[302];
	extern double Hegoal, Hegoalerr;
	extern long useHelium;

	long j, fittedSteps;
	double deviation, stepweight, sumweight;
	double Hedeviation;
	
	deviation = 0.0;
	fittedSteps = 0;
	stepweight = 0.0;

// first do age spectrum

	for (j = 1; j<= goalsteps; j++)
	{
		if (goal_loss[j] <= fitmax)  // for later use, but for now fitmax set to 1.0 in main.c
		{

//	printf("step %7d ",j);

			if (goal_skip[j] > 0)
			{
				fittedSteps = fittedSteps + 1;
				switch (fitchoice)
				{
					case 0:  // step-weighted mean percent deviation
						if (goal_loss[j] < 0.1)   // empiricially weight some early steps more given that they have more thermal relief
						{
							stepweight = 2*(goal_loss[j] - goal_loss[j-1]); // we can do this because we set goal_loss[0] to 0.0 in infiles()
						}
						else     // just use step size as weighting factor
						{
							stepweight = goal_loss[j] - goal_loss[j-1]; // we can do this because we set goal_loss[0] to 0.0 in infiles()
						}
						sumweight = sumweight + stepweight;
						deviation = deviation + stepweight*fabs(poolAge[index][j] - goal_age[j])/goal_age[j];
					break;
					case 1:  // mean percent deviation
						deviation = deviation + fabs(poolAge[index][j] - goal_age[j])/goal_age[j];		
					break;
					
					case 2:  // mswd
						deviation = deviation + 1/(goal_error[j]*goal_error[j])*pow((poolAge[index][j] - goal_age[j]),2.0);
					break;
				}
//				printf("deviation %7.3f goal-age %7.1f pred age %7.1f\n",deviation,goal_age[j],poolAge[index][j]);
			}
			else
			{
//				printf("\n");
			}
		}
	}
	assert(fittedSteps >= 1);
	if (fitchoice == 0)
	{
		deviation = deviation/sumweight;  // divide by sum of weights to get corrected weighted average
	}
	else
	{
		deviation = deviation/fittedSteps; // divide to get either simple mpd or mswd (assume d.f. = fitted steps in this case (?))	
	}

// then do helium age
	if (useHelium == 1)
	{	
		if (fitchoice < 2)  // mean percent deviation
		{
			Hedeviation = fabs(Hepool[index] - Hegoal)/Hegoal;
		}
		else                // mswd
		{
			Hedeviation = 1/(Hegoalerr*Hegoalerr)*pow((Hepool[index] - Hegoal),2.0);
		}
	}

// now do weighted average of Ar and He fits

	if (useHelium == 1)
	{
		deviation = (deviation + Heweight*Hedeviation)/(1 + Heweight);
	}
	
	if (fitchoice < 2)  // mean percent deviation
	{
		deviation = deviation*100.0;  //express mpd as percentage
	}

	return deviation;
}

void worstfit(void)
{

	extern long poolsize, WorstOne;
	extern double ModelFit[302];
	
	long ii;
	
	WorstOne = 1;
	for (ii = 2; ii <= poolsize; ii++)
	{
		if (ModelFit[ii] > ModelFit[WorstOne])
		{
			WorstOne = ii;
		}
	}
}

void bestfit(void)
{

	extern long poolsize, BestOne;
	extern double ModelFit[302];
	
	long ii;
	
	BestOne = 1;
	for (ii = 2; ii <= poolsize; ii++)
	{
		if (ModelFit[ii] < ModelFit[BestOne])
		{
			BestOne = ii;
		}
	}

	return;
}
