/* 	

		Function infiles of Arvert 4.x
		P. Zeitler, Lehigh University
		2/16/04
		
		Gets input data:
   
		(1) file domain.in (partly equivalent to Lovera's ages_me.in) 
		which has sample's diffusion and domain parameters
   
		and
   
		(2) file goal.in, which has the measured age spectrum and flags
		for whether or not to fit particular steps
		
		and
		
		(3) file crs.in, which has parameters needed for the CRS algorithm
		
		and
		
		(4) file helium.in, which has parameters for helium sample
		
		
		3/15/03 file crs.in now also contains dt and ichange parameters for Lovera routine
		
		3/30/03 program now reads in the Monte and CRS cooling- and heating-rate limits, and a flag for full or partial report
		
		2/16/04 program text now reflects change to mswd from chi-square
  	
*/

#include "arvert.h"

void infiles(void)
{
  FILE *ifp;

	extern long ndomains,goalsteps;
//	extern double temperaturelab[101],timelab[101];
	extern double E[11],D0[11],fraction[11];
	extern double goal_loss[101],goal_age[101],goal_error[101];
	extern long goal_skip[101];
	extern char info[101],suffix[10];
	extern long geometry, fitchoice;
	extern double modelduration, slab;
	extern long ttpoints;
	extern long nconstraints;
	extern double constraintTime[nodes], constraintTempmin[nodes], constraintTempmax[nodes];
	extern double heatRate, coolRate;
	extern double monteHeatRate,monteCoolRate;
	extern long CRSiterations;
	extern double amplify, fitvalue;
	extern long poolsize, subsetsize;
	extern long restart;
	extern double ichange, dt;
	extern long fullreport;
	
	extern double Hegoal, Hegoalerr;
	extern double Heact, HeDifzero;
	extern double Heradius;
	extern long mineral, useHelium;
	extern double Heweight;

	long j, proceed;
	char filename[30],shorten[30];
	
	time_t now;

//  ------------ Get inversion parameters from file <crs.in> -------------

	ifp = fopen("crs.in", "r");
/*
	j = 0;
	c = '\0';
	while (c != '\n')   // sample info
	{
	fscanf(ifp, "%c", &c);
	 if (j <= 100)
	 {
		info[j] += c;
	 }
		j++;
	}
*/
	fgets(info, MAXLINE, ifp);
	printf("          SAMPLE INFO:   %s", info);
/*	
	j = 0;
	c = '\0';
	while (c != '\n')   // file suffix... only ten characters
	{
		fscanf(ifp, "%c", &c);
	 	if (j < 10)
	 	{
			suffix[j] += c;
		}
		j++;
	}
	suffix[j-1] = '\0';
	*/
	fgets(shorten, MAXLINE, ifp);
	strncpy(suffix,shorten,strlen(shorten)-1);
	printf("          FILE SUFFIX:   %s\n", suffix);	
	// ------------- Open file to record sample and run info -------------------

	strcpy(filename,"ModelInfo.");

	now = time(NULL);
  
  fscanf(ifp, "%ld", &CRSiterations);
	printf("       CRS ITERATIONS:   %ld\n", CRSiterations);
	
  fscanf(ifp, "%lf", &modelduration);
	printf("MODEL DURATION (m.y.):   %6.1f\n", modelduration);

	assert ((modelduration > 0) && (modelduration < 4500)); //make sure times are in m.y.	
	fscanf(ifp, "%ld", &ttpoints);
	assert ((ttpoints >= 2) && (ttpoints <= 15)); //make sure arrays stay in bounds and have at least 2 to handle endpoints!
	printf("           TIME NODES:     %ld\n", ttpoints);
	printf("\n");	

	fscanf(ifp, "%ld", &nconstraints);
	assert ((nconstraints >= 2) && (nconstraints < 11)); //make sure arrays stay in bounds
	printf("CONSTRAINING BRACKETS tT:   %ld\n", nconstraints);
	
	printf("   TIME    TMIN   TMAX\n");
	for (j = 1; j <= nconstraints; j++)
	{
		fscanf(ifp, "%lf%lf%lf", &constraintTime[j], &constraintTempmin[j], &constraintTempmax[j]);
		printf(" %6.1f   %5.1f   %5.1f\n", constraintTime[j], constraintTempmin[j], constraintTempmax[j]);
		assert(constraintTempmin[j] < constraintTempmax[j]);
		assert ((constraintTime[j] <= modelduration) && (constraintTime[j] >= 0.0));
		if (j == 1)
		{
			assert(constraintTime[j] == modelduration);  // first contraint has to be at start
		}
		if (j > 1)
		{
			assert(constraintTime[j] < constraintTime[j-1]); // each successive constraint must be younger
		}
		if (j == nconstraints)
		{
			assert(constraintTime[j] == 0.0); //last constraint has to be at end
		}
	}
	printf("\n");
	
	fscanf(ifp, "%lf%lf", &monteHeatRate,&monteCoolRate); //implicit rate constraints for monte-carlo histories
	printf("MAX MONTE-CARLO HEATING RATE: %5.1f   MAX MONTE-CARLO COOLING RATE: %5.1f\n", monteHeatRate,monteCoolRate);
	assert ((monteHeatRate >= 0.0) && (monteHeatRate <= 1000)); //make sure rates are not negative or absurdly high
	assert ((monteCoolRate >= 0.0) && (monteCoolRate <= 1000));
	
	fscanf(ifp, "%lf%lf", &heatRate,&coolRate);  // implicit rate constraints for CRS histories
	printf("        MAX CRS HEATING RATE: %5.1f           MAX CRS COOLING RATE: %5.1f\n", heatRate,coolRate);
	assert ((heatRate >= 0.0) && (heatRate <= 1000)); //make sure rates are not negative or absurdly high
	assert ((coolRate >= 0.0) && (coolRate <= 1000));
	
	fscanf(ifp, "%lf", &amplify);
	printf("    CRS AMPLIFICATION FACTOR:   %4.2f\n", amplify);
	assert ((amplify >= 0.5) && (amplify <= 2)); //make sure amplify is not completely absurd
	
	fscanf(ifp, "%ld%ld", &subsetsize, &poolsize);
	printf("      SUBSET SIZE, POOL SIZE:   %ld   %ld\n", subsetsize, poolsize);
	assert(poolsize <= 300);
	assert ((subsetsize < 51) && (subsetsize <= floor(poolsize/3)) && (subsetsize >= MINSUBSET)); //make sure subset size not stupid
	printf("\n");
	
	fscanf(ifp, "%lf", &fitvalue);
	printf("           FITTING CRITERION:  %5.2f\n", fitvalue);
	assert (fitvalue >= 0); //make sure fit is reasonable
	
	fscanf(ifp, "%ld", &fitchoice);
	printf("              FITTING OPTION:   %ld  ", fitchoice);
	assert ((fitchoice == 0) || (fitchoice == 1) || (fitchoice == 2)); // 0 step-weighted mean percent 1 mean percent, 2 mswd
	switch (fitchoice)
	{
		case 0:
			printf("(step-weighted mean percent deviation)\n");
			break;
		case 1:
			printf("(mean percent deviation)\n");
			break;
		case 2:
			printf("(mswd)\n");
	}
	
	fscanf(ifp, "%ld", &geometry);
	printf("          DIFFUSION GEOMETRY:   %ld  ", geometry);
	assert ((geometry == 1) || (geometry == 2)); // 1 spheres, 2 slabs
	if (geometry == 1)
	{
		printf("(spherical)\n");
	}
	else
	{
		printf("(infinite-slab)\n");
	}
	
	fscanf(ifp, "%ld", &restart);
	printf("              RESTART OPTION:   %ld  ", restart);
	if (restart == 0)
	{
		printf("(new start from Monte-Carlo histories)\n");
		printf("\n");
	}
	else
	{
		printf("(restart using values saved in file <CRSrestart>)\n");
		printf("\n");
	}

	fscanf(ifp, "%lf", &dt);
	printf("DISCRETIZATION DELTA-TEMPERATURE: %5.1f\n", dt);
	assert ((dt >= 1.0) && (dt <= 20)); //make sure dt value is reasonable
	
	fscanf(ifp, "%lf", &ichange);
	printf("           LOVERA SERIES CUT-OFF:   %5.1e\n", ichange);
	assert ((ichange <= 0.001) && (dt >= 1.0e-8)); //make sure ichange value is reasonable

	fscanf(ifp, "%ld", &fullreport);
	printf("      FLAG TO WRITE FULL REPORTS:   %ld\n", fullreport);  // 0 means only rolling-progress resports
	assert ((fullreport == 0) || (fullreport == 1)); //make sure fullreport value is reasonable		
	printf("\n");
	fclose(ifp);

// get helium information 

	printf("           ************* Helium Info *************\n");

	ifp=fopen("helium.in","r");
	
	fscanf(ifp, "%ld", &mineral);
	assert ((mineral == 0) || (mineral == 1 ));	
	if (mineral == 1)
	{
		printf("                      Mineral:   1 - zircon\n");
	}
	else
	{
		printf("                      Mineral:   0 - apatite\n");
	}

	fscanf(ifp, "%lf", &Heact);
	printf("            ACTIVATION ENERGY:   %5.2f kcal/mol\n", Heact);
	assert ((Heact >= 20) && (Heact <= 60 ));
	
	fscanf(ifp, "%lf %lf", &Heradius,&HeDifzero);
	printf("             EFFECTIVE RADIUS: %7.2f microns     DIFZERO:  %9.3e cm2/sec\n", Heradius, HeDifzero);
	assert ((Heradius >= 25) && (Heradius <= 1000 ));
	assert(HeDifzero > 0); 	
	
	fscanf(ifp, "%lf%lf", &Hegoal,&Hegoalerr);
	printf("                  GOAL He AGE:  %6.2f Ma     ERROR IN AGE: %5.2f Ma\n", Hegoal, Hegoalerr);
	assert ((Hegoalerr > 0.0) && (Hegoal >= 0.0)); 
	
	fscanf(ifp, "%lf", &Heweight);
	printf("  WEIGHTING FACTOR FOR He AGE:  %6.2f\n", Heweight);
	assert ((Heweight >= 0.0) && (Heact <= 500));	
	 
	fscanf(ifp, "%ld", &useHelium);
	assert ((useHelium == 0) || (useHelium == 1 ));	
	if (useHelium == 1)
	{
		printf("HELIUM AGE USED AS CONSTRAINT:    1 - yes\n");
	}
	else
	{
		printf("HELIUM AGE USED AS CONSTRAINT:    0 - no\n");
	}

	fclose(ifp);

	printf("-------------------------------------------------------------------------------\n");
	printf("Values OK?\n");
	printf("  0 -- Proceed\n");
	printf("  1 -- Abort model  --> ");
	scanf("%ld", &proceed);
	printf("-------------------------------------------------------------------------------\n");
	if (proceed == 0)
	{
		printf("\n");
		printf(">>> cmd-P to pause (return to continue); esc or cntrl-C to abort (then Quit) <<<\n");
		printf("\n");
		printf("Model starting...\n");
		printf("\n");
	}
	else
	{
		printf("MODEL ABORTED\n");
		exit(j);
	}

//  ------------ Get diffusion parameters from file <domains.in> -------------

	ifp = fopen("domains.in", "r");
	fscanf(ifp, "%ld", &ndomains);   // number of domains
	assert(ndomains > 0 && ndomains < 11);
	for (j = 1; j <= ndomains; j++)
	{
		fscanf(ifp, "%lf%lf%lf", &E[j], &D0[j], &fraction[j]);
		D0[j] = exp(2.3025851 * D0[j]);  // convert log10 D0 to true values
		D0[j] = D0[j] * 31557600e6;    // 1/sec to 1/m.y.
		
		slab = 1;
		if (geometry == 2)
		{
			slab = 4;    // account for extra coefficient needed for slabs
		}	
	}
	fclose(ifp);

//  ------------ Get goal spectrum data from file <goal.in> -------------

	ifp = fopen("goal.in", "r");
	fscanf(ifp, "%ld", &goalsteps);   // number of goal-spectrum steps
	assert(goalsteps > 0 && goalsteps < 101);
	for (j = 1; j <= goalsteps; j++)
	{
		fscanf(ifp, "%lf%lf%lf%ld", &goal_loss[j], &goal_age[j], &goal_error[j], &goal_skip[j]);
		assert ((goal_skip[j] == 0) || (goal_skip[j] == 1 ));	
	}
	goal_loss[0] = 0.0; // set this to make it easier to calculate step losses with simple loop
	fclose(ifp);

	
//	fscanf(ifp, "%d", &goalsteps);   // number of goal-spectrum steps
//	for (j = 1; j <= goalsteps; j++)
//	{
//		printf("%d  %f  %f  %f  %d\n", j, goal_loss[j], goal_age[j], goal_error[j], goal_skip[j]);
//	}
	fclose(ifp);
	
	return;
}
