/* 	

		Function initialize() of Arvert 4.x
		P. Zeitler, Lehigh University
		2/16/04
		
		Sets and calculates some parameters.
				
		Operates on program globals (which most of the main variables are...)
		
		2/16/04: fixed small bug in 4-node option: need to fudge nodes _into_ spectrum's range
		to try and get away from just modeling one linear segment.
  	
*/

#include "arvert.h"

void initialize(void)
{
	extern long geometry;
	extern double b;
	extern long RanSeedZero;
	extern double MonteTime[nodes];
  extern double lowerlimit[nodes], upperlimit[nodes];
  extern long ttpoints;
  extern double modelduration;
  extern long nconstraints;
  extern double constraintTime[nodes], constraintTempmin[nodes], constraintTempmax[nodes];
  extern double goal_loss[101],goal_age[101];
  extern double maxgoodgoalage, mingoodgoalage, mingoodgoalloss, maxgoodgoalloss;
	extern long goal_skip[101];
	extern long goalsteps;
	extern double stopping_distance;
	extern long mineral;
	extern double Heradius;
	extern long useHelium;
	extern double Hegoal;
	
	long seed,i,j;
	long step_flag;
	double dummy;
	double dage;
	
//	long pause;
	
// MUST be 1 for spheres, 2 for slabs because used in code below
		if (geometry == 1) 
	{
			b = 6.0;    // spheres (geometry is 1)
	}
	else 
	{
			b = 8.0;    // infinite slabs (geometry is 2)
	}
	
// a seed for the random-number generator; use -ABS() to ensure snafu with time doesn't hurt seed
	seed = -abs(time(NULL));

// seed := -10;  DEBUG  use this line to lock in the random sequence for every run

	dummy = ran3(seed); // subsequent calls to RAN3(RanSeedZero) get successive values in pseudo-random series
	RanSeedZero = 10;  // this is just a dummy parameter that should be positive


// Shape constraints into xxxerLimit[] array
// Array indices for upperlimit[] and lowerlimit[] map to those of timein[];

// Constraint at start:
// Lovera routine REQUIRES initial cooling before any reheating might occur. So even
// if there are no specified constraints, must have a minumum temp greater than Tc of largest domain.
// For now be lazy and make it 450 C.

	if (constraintTempmin[1] < 450)  //force first minimum constraint to be high
	{
		constraintTempmin[1] = 450;
		if (constraintTempmin[1] > constraintTempmax[1])  //make sure coercion doesn't foul max constraint
		{
			constraintTempmax[1] = 500;
		}
	}
	
// Extract time nodes from input age spectrum
// For legacy reasons, time arrays are 1-based!

  // first, parse goal spectrum to get minimum and maximum goal-spectrum ages that are "good" (i.e., used in fitting)
  // given that we generally expect spectra to rise, we can avoid messy search by just going up from start and down
  // from end and find first step flagged as "good". Only problem with this is if someone were to model a very odd
  // sample with differing activation energies in domains such that ages dropped at high release. But code will
  // still function in this case, even if true max age won't be found for use in making time nodes.
  
  // goal arrays are 1-based
  
  // Do not laugh at following routine, written while brain dead. It's verbose but it works. 
  
  step_flag = -1;
  i = 0;
  
  while (step_flag < 0)
  {
  	i++;
  	assert(i <= 101);  // trap infinite loop if user screwed up and no steps are flagged as good!
  	if (goal_skip[i] > 0)
  	{
  		step_flag = 1;
  		mingoodgoalage = goal_age[i];
  		mingoodgoalloss = goal_loss[i]; 
  	}
  }
  
  step_flag = -1;
  i = goalsteps+1;
  
  while (step_flag < 0)
  {
  	i--;
  	assert(i > 0);  // trap infinite loop if user screwed up and no steps are flagged as good!
  	if (goal_skip[i] > 0)
  	{
  		step_flag = 1;
  		maxgoodgoalage = goal_age[i];
  		maxgoodgoalloss = goal_loss[i]; 
  	}
  }
  
 	dage = maxgoodgoalage - mingoodgoalage;
 
  if (dage == 0.0)  //probably someone is inverting a flat spectrum (!)
  {
  	maxgoodgoalage = 1.10*mingoodgoalage;
  }
  assert(maxgoodgoalage <= 0.8*modelduration);  // insist on some headroom in model, partly to allow for later repairs to interpolation of time nodes
  assert(mingoodgoalage >= 0.0);
	assert(maxgoodgoalage > mingoodgoalage);

	// next, deal with model endpoints
	MonteTime[1] = modelduration;
	MonteTime[ttpoints] = 0.0;   //worry in the future about option to focus model on intervals; as it is,
	// the spectra we measure extend to 0.0; would have to subtract static shift from goal spectrum
	// to handle intervals and right now I'm too lazy
	
	
	// how to deal with helium. if he age is < 0.8 of min spec age, work to accomodate it, so cheeck for this.
	// then, (tedious) for each node number, try and distribute a few nodes closer to Hegoal, if user
	//has chosen option to use He constraint. actually, compare Hegoal with min good spec age, and adjust
	//accordingly, being careful not to get nodes out of order.
	
	
	
	// now deal with remaining "internal" ttpoints
		// including another two endpoints: max and min values of spectrum
		// and have to deal with special cases when ttpoints is low (less than 5).
	
	// if ttpoints is 3, can only park a time node one place, might as well be certain it's within age spectrum to
	// focus efforts of inversion in range of observed ages
	
	if (ttpoints == 3)
	{
		MonteTime[2] = (maxgoodgoalage + mingoodgoalage)/2.0;
	}
	
	// if ttpoints is 4, if we make two internal nodes be the max and min ages, we really only cover spectrum
	// with one linear segment. To allow for some non-linear cooling, cheat end nodes into spectrum's range, a little.
	// amount is arbitary and the degree to which this helps will depend on way total model space straddles
	// active portion during which ages are set. Try 10% for starters.
	
	if (ttpoints == 4)
	{
		MonteTime[2] = maxgoodgoalage - 0.10*(dage);
		MonteTime[3] = mingoodgoalage + 0.10*(dage);		
	}
	
	if (ttpoints == 5) // we have one "free" time node--cheat it towards young side within spectrum's span
	{
		MonteTime[2] = maxgoodgoalage + 0.10*(modelduration - maxgoodgoalage);
		MonteTime[3] = mingoodgoalage + 0.20*(dage);
		MonteTime[4] = mingoodgoalage - 0.10*mingoodgoalage;
	}
	
	if (ttpoints == 6) // we have two "free" time nodes--give one to earlier part of model
	{
		MonteTime[2] = maxgoodgoalage + 0.6*(modelduration - maxgoodgoalage);
		MonteTime[3] = maxgoodgoalage - 0.15*(dage);
		MonteTime[4] = mingoodgoalage + 0.20*(dage);
		MonteTime[5] = mingoodgoalage - 0.15*mingoodgoalage;
	}
	
	if (ttpoints == 7) // we have three "free" time nodes.
	{
		MonteTime[2] = maxgoodgoalage + 0.6*(modelduration - maxgoodgoalage);
		MonteTime[3] = maxgoodgoalage + 0.10*(modelduration - maxgoodgoalage);
		MonteTime[4] = maxgoodgoalage - 0.15*(dage);
		MonteTime[5] = mingoodgoalage + 0.20*(dage);
		MonteTime[6] = mingoodgoalage - 0.15*mingoodgoalage;
	}
	
	if (ttpoints == 8) // we have four "free" time nodes.
	{
		MonteTime[2] = maxgoodgoalage + 0.6*(modelduration - maxgoodgoalage);
		MonteTime[3] = maxgoodgoalage + 0.10*(modelduration - maxgoodgoalage);
		MonteTime[4] = maxgoodgoalage - 0.15*(dage);
		MonteTime[5] = mingoodgoalage + 0.20*(dage);
		MonteTime[6] = mingoodgoalage - 0.15*mingoodgoalage;
		if ((useHelium == 1) && (Hegoal <= 0.65*mingoodgoalage))
		{
			MonteTime[7] = Hegoal;		
		}
		else
		{
			MonteTime[7] = mingoodgoalage - 0.35*mingoodgoalage;
		}
	}
	
	if (ttpoints == 9) // we have five "free" time nodes.
	{
		MonteTime[2] = maxgoodgoalage + 0.6*(modelduration - maxgoodgoalage);
		MonteTime[3] = maxgoodgoalage + 0.10*(modelduration - maxgoodgoalage);
		MonteTime[4] = maxgoodgoalage - 0.15*(dage);
		MonteTime[5] = mingoodgoalage + 0.5*(dage);
		MonteTime[6] = mingoodgoalage + 0.20*(dage);
		MonteTime[7] = mingoodgoalage - 0.15*mingoodgoalage;
		if ((useHelium == 1) && (Hegoal <= 0.65*mingoodgoalage))
		{
			MonteTime[8] = Hegoal;		
		}
		else
		{
			MonteTime[8] = mingoodgoalage - 0.35*mingoodgoalage;
		}
	}	
	
	if (ttpoints == 10) // we have six "free" time nodes.
	{
		MonteTime[2] = maxgoodgoalage + 0.6*(modelduration - maxgoodgoalage);
		MonteTime[3] = maxgoodgoalage + 0.10*(modelduration - maxgoodgoalage);
		MonteTime[4] = maxgoodgoalage - 0.15*(dage);
		MonteTime[5] = mingoodgoalage + 0.65*(dage);
		MonteTime[6] = mingoodgoalage + 0.35*(dage);
		MonteTime[7] = mingoodgoalage + 0.15*(dage);
		MonteTime[8] = mingoodgoalage - 0.15*mingoodgoalage;
		if ((useHelium == 1) && (Hegoal <= 0.65*mingoodgoalage))
		{
			MonteTime[9] = Hegoal;		
		}
		else
		{
			MonteTime[9] = mingoodgoalage - 0.35*mingoodgoalage;
		}
	}		
	
	if (ttpoints == 11) // we have seven "free" time nodes.
	{
		MonteTime[2] = maxgoodgoalage + 0.6*(modelduration - maxgoodgoalage);
		MonteTime[3] = maxgoodgoalage + 0.10*(modelduration - maxgoodgoalage);
		MonteTime[4] = maxgoodgoalage - 0.15*(dage);
		MonteTime[5] = mingoodgoalage + 0.65*(dage);
		MonteTime[6] = mingoodgoalage + 0.35*(dage);
		MonteTime[7] = mingoodgoalage + 0.15*(dage);
		MonteTime[8] = mingoodgoalage - 0.15*mingoodgoalage;
		if ((useHelium == 1) && (Hegoal <= 0.55*mingoodgoalage))
		{
			MonteTime[9] = (Hegoal + 0.85*mingoodgoalage)/2;
			MonteTime[10] = 0.9*Hegoal;
		}
		else
		{
			MonteTime[9] = mingoodgoalage - 0.30*mingoodgoalage;
			MonteTime[10] = mingoodgoalage - 0.45*mingoodgoalage;
		}		
	}			
	
	if (ttpoints == 12) // we have eight "free" time nodes.
	{
		MonteTime[2] = maxgoodgoalage + 0.6*(modelduration - maxgoodgoalage);
		MonteTime[3] = maxgoodgoalage + 0.10*(modelduration - maxgoodgoalage);
		MonteTime[4] = maxgoodgoalage - 0.15*(dage);
		MonteTime[5] = mingoodgoalage + 0.65*(dage);
		MonteTime[6] = mingoodgoalage + 0.35*(dage);
		MonteTime[7] = mingoodgoalage + 0.20*(dage);	
		MonteTime[8] = mingoodgoalage + 0.10*(dage);
		MonteTime[9] = mingoodgoalage - 0.15*mingoodgoalage;
		if ((useHelium == 1) && (Hegoal <= 0.55*mingoodgoalage))
		{
			MonteTime[10] = (Hegoal + 0.85*mingoodgoalage)/2;
			MonteTime[11] = 0.9*Hegoal;
		}
		else
		{
			MonteTime[10] = mingoodgoalage - 0.30*mingoodgoalage;
			MonteTime[11] = mingoodgoalage - 0.45*mingoodgoalage;
		}	
	}		
	
	if (ttpoints == 13) // we have nine "free" time nodes.
	{
		MonteTime[2] = maxgoodgoalage + 0.6*(modelduration - maxgoodgoalage);
		MonteTime[3] = maxgoodgoalage + 0.10*(modelduration - maxgoodgoalage);
		MonteTime[4] = maxgoodgoalage - 0.15*(dage);
		MonteTime[5] = mingoodgoalage + 0.80*(dage);
		MonteTime[6] = mingoodgoalage + 0.55*(dage);
		MonteTime[7] = mingoodgoalage + 0.35*(dage);
		MonteTime[8] = mingoodgoalage + 0.20*(dage);
		MonteTime[9] = mingoodgoalage + 0.10*(dage);
		MonteTime[10] = mingoodgoalage - 0.15*mingoodgoalage;
		if ((useHelium == 1) && (Hegoal <= 0.55*mingoodgoalage))
		{
			MonteTime[11] = (Hegoal + 0.85*mingoodgoalage)/2;
			MonteTime[12] = 0.9*Hegoal;
		}
		else
		{
			MonteTime[11] = mingoodgoalage - 0.30*mingoodgoalage;
			MonteTime[12] = mingoodgoalage - 0.45*mingoodgoalage;
		}	
	}			
		
	if (ttpoints == 14) // we have ten "free" time nodes.
	{
		MonteTime[2] = maxgoodgoalage + 0.75*(modelduration - maxgoodgoalage);
		MonteTime[3] = maxgoodgoalage + 0.35*(modelduration - maxgoodgoalage);
		MonteTime[4] = maxgoodgoalage + 0.10*(modelduration - maxgoodgoalage);
		MonteTime[5] = maxgoodgoalage - 0.15*(dage);
		MonteTime[6] = mingoodgoalage + 0.80*(dage);
		MonteTime[7] = mingoodgoalage + 0.55*(dage);
		MonteTime[8] = mingoodgoalage + 0.35*(dage);
		MonteTime[9] = mingoodgoalage + 0.20*(dage);
		MonteTime[10] = mingoodgoalage + 0.10*(dage);
		MonteTime[11] = mingoodgoalage - 0.15*mingoodgoalage;
		if ((useHelium == 1) && (Hegoal <= 0.55*mingoodgoalage))
		{
			MonteTime[12] = (Hegoal + 0.85*mingoodgoalage)/2;
			MonteTime[13] = 0.9*Hegoal;
		}
		else
		{
			MonteTime[12] = mingoodgoalage - 0.30*mingoodgoalage;
			MonteTime[13] = mingoodgoalage - 0.45*mingoodgoalage;
		}	
	}		

	if (ttpoints == 15) // we have eleven "free" time nodes.
	{
		MonteTime[2] = maxgoodgoalage + 0.75*(modelduration - maxgoodgoalage);
		MonteTime[3] = maxgoodgoalage + 0.40*(modelduration - maxgoodgoalage);
		MonteTime[4] = maxgoodgoalage + 0.15*(modelduration - maxgoodgoalage);
		MonteTime[5] = maxgoodgoalage - 0.10*(dage);
		MonteTime[6] = mingoodgoalage + 0.80*(dage);
		MonteTime[7] = mingoodgoalage + 0.55*(dage);
		MonteTime[8] = mingoodgoalage + 0.35*(dage);
		MonteTime[9] = mingoodgoalage + 0.15*(dage);
		MonteTime[10] = mingoodgoalage + 0.05*(dage);
		if (useHelium == 1 && Hegoal <= 0.55*mingoodgoalage)
		{
			MonteTime[11] = Hegoal + 0.90*(mingoodgoalage - Hegoal);
			MonteTime[12] = Hegoal + 0.65*(mingoodgoalage - Hegoal);
			MonteTime[13] = Hegoal + 0.25*(mingoodgoalage - Hegoal);
			MonteTime[14] = 0.8*Hegoal;
		}
		else
		{
			MonteTime[11] = mingoodgoalage - 0.10*mingoodgoalage;
			MonteTime[12] = mingoodgoalage - 0.20*mingoodgoalage;
			MonteTime[13] = mingoodgoalage - 0.30*mingoodgoalage;
			MonteTime[14] = mingoodgoalage - 0.45*mingoodgoalage;
		}	
	}			
	
/*
printf("tnode     Time      mingoalage   maxgoalage\n");
for (i=1;i<=ttpoints;i++)
{
  printf(" %d    %f    %f    %f\n", i,MonteTime[i],mingoodgoalage,maxgoodgoalage);
}
*/
assert(MonteTime[2] < modelduration); //one last check that no time nodes slipped past model duration

// get upper and lower temperature limits at each time node

	upperlimit[1] = constraintTempmax[1];
	lowerlimit[1] = constraintTempmin[1];
	upperlimit[ttpoints] = constraintTempmax[nconstraints];
	lowerlimit[ttpoints] = constraintTempmin[nconstraints];	
	for (i = 2; i <= ttpoints - 1; i++)
	{
		j = 1;
		do
		{
			j++;
		}
		while (MonteTime[i] < constraintTime[j]);
		
		j = j - 1;
		
		upperlimit[i] = constraintTempmax[j] - (constraintTime[j] - MonteTime[i])/(constraintTime[j] - constraintTime[j+1])*(constraintTempmax[j] - constraintTempmax[j+1]);
// printf("%d   %f   %f   %f   %f   %f\n",j,MonteTime[i],constraintTime[j],constraintTime[j+1],constraintTempmax[j],constraintTempmax[j+1]);
		lowerlimit[i] = constraintTempmin[j] - (constraintTime[j] - MonteTime[i])/(constraintTime[j] - constraintTime[j+1])*(constraintTempmin[j] - constraintTempmin[j+1]);
	}

//printf("tnode   Time   mincon   maxcon\n");
//for (i=1;i<=ttpoints;i++)
//{
//  printf(" %d    %f    %f    %f\n", i,MonteTime[i],lowerlimit[i],upperlimit[i]);
//}

//	printf("PAUSED for input\n");
//	printf("  QUIT to abort model  --> ");
//	scanf("%d", &i);

// do some things for helium()

	if (useHelium == 1)
	{
		if (mineral == 1)
		{
			stopping_distance =   16.65e-4; // mean stopping distance in cm for 238-U alpha-recoil, zircon
		}
		else
		{
			stopping_distance =   19.68e-4; // mean stopping distance in cm for 238-U alpha-recoil, apatite	
		}
				
		Heradius = Heradius /1e4;  // convert microns to cm
	}

	return;
}