/* 	

		Function lovera() of Arvert 4.x
		P. Zeitler, Lehigh University
		7/5/00
		
   C translation of Oscar Lovera's program ages.f (translated through intermediate
   Pascal translation, also by Zeitler!). Somehow it works.
   
   Handles heating as well as cooling as long as, after the model start
   there is complete cooling from well above closure to below closure
   
   Input files are file domains.in, partly equivalent to Lovera's ages_me.in, 
   which has sample diffusion and domain parameters
   
   and
   
   file heatsched.in, which has the lab temperature history
  	
*/

#include "arvert.h"

void lovera(void)
{
	extern long geometry;
	extern double timein[nodes],temperaturein[nodes],timelab[101], temperaturelab[101];
	extern double E[11],D0[11],fraction[11],b, slab;
	extern long ndomains, labsteps;
	extern double bulkloss39[101], age[101];
	extern double dt, ichange;
	extern long ttpoints;
	

	double *xi, *temperature ,*time;
	double r39[101], r40[101], tra40[101];
	double sra40[101], sra39[101], scura[101];
	double c40, c39wd;
	
	long j, ns;
	
//	long dummy;

// following are from chist procedure of Lovera

	double Tdiff, deltaTemp, deltaTime;
	long newpoints;
	long ij, nt;
	
// following are from geosec procedure of Lovera
	double zit, avtemperature, a1, uplus, cam, sum;
	double	*dzita, *d;
	long	n, jj, nj;
	long	madj, mfake, m;

// following are from lab procedure of Lovera
	double  kpie, prev_r40, prev_r39, change39, change40, zii, ab;
	double zita[101];
	long jjj, k;
	
//  clock_t started, ended;
//  double elapsed;

	xi = calloc(50003, sizeof(double));
	temperature = calloc(nch, sizeof(double));
	time = calloc(nch, sizeof(double));
	dzita = calloc(nch, sizeof(double));
	d = calloc(nch, sizeof(double));

//  ------------ Set some needed parameters -------------

	c40 = 1.0 - exp(-xlambd * timein[1]);
	c39wd = exp(-xlambd * timein[1]);
	
	ns = 1;
	temperature[1] = temperaturein[1];
	time[1] = timein[1];
	
// discretize the input thermal history--new PZ algorithm
 
	for (k = 1; k <= ttpoints - 1; k++)
	{
		Tdiff = temperaturein[k] - temperaturein[k + 1];
		if (fabs(Tdiff) <= dt)  //if delta-T small have at least number time and temp points given input history
		{
			ns = ns + 1;
			temperature[ns] = temperaturein[k + 1];
			time[ns] = timein[k + 1];
		}
		else
		{
			newpoints = (long) floor(fabs(Tdiff)/dt + 0.5) + 1;
			deltaTemp = Tdiff/newpoints;
			deltaTime = (timein[k] - timein[k+1])/newpoints;
			for (ij = 1; ij <= newpoints; ij++)
			{
			temperature[ij + ns] = temperaturein[k] - ij*deltaTemp;
			time[ij + ns] = timein[k] - ij*deltaTime;
			}
			ns = ns + newpoints;
		}
		assert(ns <= nch - 2);
	}


// fix last point to that of input function (this avoids problems with interpolated non-rational times ending slightly negative)
	temperature[ns] = temperaturein[ttpoints];
	time[ns] = timein[ttpoints];
	
/*
	for (ij = 1; ij <= ns; ij++)  // DEBUG -- Print out discretized time-temps
	{
		if ((ij < 20) || (ij > ns-19))
		{
			printf(" %d  %10.6f  %5.1f\n",ij,time[ij],temperature[ij]);
		}
	}

	printf("PAUSED for input\n");
	printf("  QUIT to abort model  --> ");
	scanf("%d", &ij);
*/	

//	printf("Finished making discretizing what appears to be a legal temperature history....\n");
//	printf("\n");

	for (j = 0; j <= labsteps; j++) // added this-- Lovera's code did not initialize arrays to zero for summation }
	{
		tra40[j] = 0.0;
		bulkloss39[j] = 0.0;
	}


// -------------------------- MAIN LOOP ------------------------------------

//	started = clock();
	for (j = 1; j <= ndomains; j++)   // calculate the age spectrum for each domain
	{
	
// printf("Starting to process domain: %2d\n",j);
    // following code is Lovera's GEOSEC procedure
		zit = 0.0;
		for (jj = 1; jj <= ns; jj++)
		{
			avtemperature = 273.15 + (temperature[jj + 1] + temperature[jj]) / 2; //include conversion to K
			d[jj] = D0[j]/slab * exp(-E[j] / R / avtemperature);
		}

		dzita[ns] = 0.0;

		for (nj = 2; nj <= ns; nj++)
		{
			jj = nj - 1;
			dzita[ns - jj] = dzita[ns + 1 - jj] + fabs(time[ns + 1 - jj] - time[ns - jj]) * d[ns - jj];
		}

// Computation of HXI(M)

		for (m = 0; m <= 49; m++)
		{
			sum = 0.0;
			for (n = 1; n <= ns-1; n++)
			{
				a1 = pow(((m*geometry + 1) * pi),2.0) * d[n] - xlambd; 
				uplus = pow(((m*geometry + 1) * pi),2.0) * dzita[n + 1] + xlambd * (time[1] - time[n + 1]);
				if (uplus <= euplus) // was 50
				{
					if ((a1 * fabs(time[n + 1] - time[n])) > ecam)
					{
						cam = 1.0;
					}
					else
					{
						cam = 1.0 - exp(-a1 * fabs(time[n + 1] - time[n]));
					}       // if-a1 conditional
					sum = sum + d[n] * pow(((m*geometry + 1) * pi),2) / a1 * cam * exp(-uplus);
				}       // end of if-uplus conditional
			}        // end for n loop
			xi[m+1] = sum;
// printf("%d %f\n",m+1,xi[m+1]);
		}        // end of for m loop

		for (m = 1; m <= 310; m++)
		{
			mfake = m*5 + 49;
			sum = 0.0;
			for (n = 1; n <= ns-1; n++)
			{
				a1 = pow(((mfake*geometry + 1) * pi),2.0) * d[n] - xlambd;
				uplus = pow(((mfake*geometry + 1) * pi),2.0) * dzita[n + 1] + xlambd * (time[1] - time[n + 1]);
				if (uplus <= euplus)  // was 50
				{
					if ((a1 * fabs(time[n + 1] - time[n])) > ecam)
					{
							cam = 1.0;
					}
					else
					{
							cam = 1.0 - exp(-a1 * fabs(time[n + 1] - time[n]));
					}     // if-a1 conditional
					sum = sum + d[n] * pow(((mfake*geometry + 1) * pi),2.0) / a1 * cam * exp(-uplus);
				}     // if-uplus conditional
			}      // end for n loop
			xi[mfake+1] = sum;
			
			for (madj = 1;madj <= 4;madj++)
			{
				xi[mfake+1-madj] = xi[mfake+1] - (xi[mfake+1] - xi[mfake+1-5])/5*madj;
			}
// printf("%d %f\n",mfake+1,xi[mfake+1]);
		}         // end  for m loop

		for (m = 1; m <= 242; m++)
		{
			mfake = m*200 + 1599;
			sum = 0.0;
			for (n = 1; n <= ns-1; n++)
			{
				a1 = pow(((mfake*geometry + 1) * pi),2.0) * d[n] - xlambd;
				uplus = pow(((mfake*geometry + 1) * pi),2.0) * dzita[n + 1] + xlambd * (time[1] - time[n + 1]);
				if (uplus <= euplus) // was 50
				{
					if ((a1 * fabs(time[n + 1] - time[n])) > ecam)
					{
						cam = 1.0;
					}
					else
					{
						cam = 1.0 - exp(-a1 * fabs(time[n + 1] - time[n]));
					}       //  end if-a1 conditional
					sum = sum + d[n] * pow(((mfake*geometry + 1) * pi),2.0) / a1 * cam * exp(-uplus);
				}       //  end if-uplus conditional
			}        //  end for n loop
			xi[mfake+1] = sum;
			for (madj = 1;madj <= 199;madj++)
			{
				xi[mfake+1-madj] = xi[mfake+1] - (xi[mfake+1] - xi[mfake+1-200])/200*madj;
			}
// printf("%d %f\n",mfake+1,xi[mfake+1]);
		}   // end  for m loop

//	printf("made it to LAB procedure ");

//  following code is Lovera's LAB procedure

		zii= 0.0;			

		for (nt = 1;nt <= labsteps;nt++)
		{
			zita[nt] = zii + D0[j]/slab * timelab[nt] * exp(-E[j] / R / temperaturelab[nt]);
			zii = zita[nt];
//			printf("%d  %e\n",nt,zii);
		}

		for (jjj = 1;jjj <= labsteps;jjj++)
		{
			r40[jjj] = 0.0;
			r39[jjj] = 0.0;
			k = -1;
			change39 = change40 = 1.0;
		
			prev_r40 = 0.0;
			prev_r39 = 0.0;

// following line is original for pkz Lovera
			while (((change39 > ichange) || (change40 > ichange)) && (k < 49999))   // loop only until values stop changing
//			while ((k < 1000))   // loop only until values stop changing

// following are test lines
//			while (k < 5000)
//	for (k = 0; k < ntest; k++)
//			while ((k < 49999))
			{
				k = k + 1;
				kpie = pow(((k*geometry + 1) * pi),2.0);
//printf("kpie %d  %d   %d  %f\n",k,geometry,(k*geometry + 1),kpie);
				if (zita[jjj] * kpie > elab)    //    was 100
				{
					ab = 0.0;
				}
				else
				{
					ab = exp(-zita[jjj] * kpie) / kpie;
					r39[jjj] = r39[jjj]  + ab;
				}
				r40[jjj] = r40[jjj] + (c40 - 1 + xi[k+1]) * (1.0 / kpie - ab);
//				printf("%d %d %e %e\n",j,k+1,xi[k+1],ab);
				change40 = (r40[jjj] - prev_r40) / r40[jjj];
				prev_r40 = r40[jjj];
				change39 = (r39[jjj] - prev_r39) / r39[jjj];
				prev_r39 = r39[jjj];
			}
				r40[jjj] = b * r40[jjj];
				r39[jjj] = 1.0 - b*r39[jjj];
		}
//	printf("k is  %d and change is %5.11f\n", k,change39);
		for (k = 1; k <= labsteps;k++)
		{
			tra40[k] = fraction[j] * r40[k] + tra40[k];
			bulkloss39[k] = fraction[j] * r39[k] + bulkloss39[k];
		}
	}   //  of main loop in j

	for (j = 1; j <= labsteps;j++)      // final data reduction }
	{
		sra40[j] = tra40[j] - tra40[j - 1];
		sra39[j] = bulkloss39[j] - bulkloss39[j - 1];
		if ((sra40[j] <= 0.0) || (sra39[j] <= 0.0))
		{
			age[j] = 0.0;
		}
		else
		{
			scura[j] = sra40[j] / sra39[j];
			age[j] = log(1.0 + scura[j] / c39wd) / xlambd;
		}
	}
/*
	printf("\n");
	for (j = 1; j <= labsteps;j++)
	{
		printf("%d  %7.4f  %6.1f\n",j,bulkloss39[j],age[j]);
	}
 */
//	ended = clock();

//	printf("********* routine Lovera finished with no worries! **********\n");

//	elapsed = (double) (ended - started);

//	printf("   (%5.2f seconds for core loop)\n",elapsed/60.0);
	free(xi);
	free(temperature);
	free(time);
	free(dzita);	
	free(d);
	return;
}