#include "arvert.h"
#define SIZE FILENAME_MAX
#define READ_OR_WRITE  0x0 /* fake a UNIX mode */
// Unfortunately, for legacy reasons this code uses a mix of zero and one-based arrays, so be careful!

// Global variables
	long labsteps,ndomains,goalsteps;
	double temperaturelab[101],timelab[101];
	double E[11],D0[11],fraction[11];
	double goal_loss[101],goal_age[101],goal_error[101];
	long goal_skip[101];
	char info[101],suffix[10],isuffix[5],filename[30],finalfilename[30],recfilename[30],extent[3];
	
	double timein[nodes],temperaturein[nodes]; //T-t history, for use by Monte and CRS routines when talking to lovera()
	double MonteTime[nodes], MonteTemp[nodes];
	double Time_at_node[nodes];
	long RanSeedZero;
	double lowerlimit[nodes], upperlimit[nodes];
	double ModelFit[302];
	long WorstOne,BestOne;

	long restart;

	double b,slab;
	
	double modelduration;
	long nconstraints;
	double constraintTime[nodes], constraintTempmin[nodes], constraintTempmax[nodes];
	double heatRate, coolRate;
	double monteHeatRate,monteCoolRate;
	long ttpoints;
	long CRSiterations;
	double amplify, fitvalue;
	long geometry;
	long fitchoice;
	long poolsize, subsetsize, CRS_selection;
	double bulkloss39[101], age[101];
	double fitmax;
	long CRSselections[302];
	double ichange, dt;
	long fullreport;
	double maxgoodgoalage, mingoodgoalage, mingoodgoalloss, maxgoodgoalloss;
	long goodHistory, CRSattempts;
	double avTemp[nodes], avAge[101], lowEnvelope[nodes], highEnvelope[nodes];
	
// for Helium machinations...
	double Heage, Hegoal, Hegoalerr;
	double Heact, HeDifzero;
	double Heradius, stopping_distance;
	long mineral, useHelium;
	double Heweight, Hepool[302];
	double avHeage;
	
	long j;

int main (int argc, const char * argv[]) {

	FILE *ifp, *ofp;
	long i, ijj, row, imonte, iCRS;
	long totCRS, nowCRS;
	double **poolAge, **poolTemp, **poolTime;
	
	time_t now;
	long duration;
	long CRSattempts;
	double elapsed, runrate;
//	double report_temp[21];

	char newFolder[SIZE], inFolder[SIZE]; // PORT-ISSUE      
		
	getcwd( inFolder, SIZE ); // PORT-ISSUE pathname call should work but check implementation on your system
	// this call gets pathname of current working directory (e.g., where Arvert is running)

	fitmax = 1.0;   // work into more complex routine with user options, later
  	
	startup();      // write a simple text header
	
	
	now = time(NULL);
	printf("-------------------------------------------------------------------------------\n");
	printf("                          %s",ctime(&now));
	printf("-------------------------------------------------------------------------------\n");

	infiles();      // read the input files
	initialize();	  // brew up a bunch of parameters

	heatsched();  // create a heating schedule based on domain parameters and 39-losses in measured spectrum

// allocate large arrays (NOTE: 0 based, so need one extra to match old algorithms)
// also need one additional extra because use it for newest CRS history

	poolAge = calloc(poolsize+2, sizeof(double *));	
	poolTemp = calloc(poolsize+2, sizeof(double *));
	poolTime = calloc(poolsize+2, sizeof(double *));
	
	for (row = 1; row <= poolsize+2; row++)
	{
		poolAge[row] = calloc(labsteps+1, sizeof(double));
		poolTemp[row] = calloc(nodes, sizeof(double));
		poolTime[row] = calloc(nodes, sizeof(double));
	}
	printf("...inputs and initialization done; large arrays allocated.\n");
	printf("\n");

// Make starting pool of monte-carlo histories
 if (restart == 0)
    {
    	totCRS = 0;  // we're starting a new model from scratch

    	ifp = fopen("CRScount.in", "w");
			fprintf(ifp, "%ld", totCRS);   // make sure counter file is zeroed for new run
			fclose(ifp);
			
   		printf("Starting to make monte-carlo histories...\n");
			for (imonte = 1;imonte<= poolsize;imonte++)
			{
				monteTt();
				
		// place new monte carlo history into main CRS pool
				for (ijj = 1;ijj <= ttpoints;ijj++)
				{
					poolTemp[imonte][ijj] = temperaturein[ijj];
					poolTime[imonte][ijj] = timein[ijj];
				}
				lovera();
				if (useHelium == 1)
				{
					helium();			
					Hepool[imonte] = Heage;
				}

				for (ijj = 1; ijj <= labsteps; ijj++)
				{
					poolAge[imonte][ijj] = age[ijj];
				}
			}

			sort(poolAge, poolTemp, poolTime);
			averages(poolAge, poolTemp);

// Change directory in preparation for writing intermediate results files 

// PORT-ISSUE - make sure these standard calls work for your system

			j = 0;	
			do
			{
				if (j == 0)
				{
					sprintf(newFolder,"RESULTS-%s", suffix);		
				}
				else     // don't overwrite older folder, so rename with digit
				{
					sprintf(newFolder,"RESULTS-%s %ld", suffix, j);
				}
				j++;
			} while (mkdir(newFolder, 0755 ) == -1); 

		/* change to new folder */

			if( chdir( newFolder) )
			{
					printf("\nCannot change to new folder\n");
					exit(EXIT_FAILURE); 
			}

		// write out monte-carlo time-temperature data, in tab-delimited format	
			strcpy(extent,".xls");
			strcpy(filename,"MONTEtT_");

			strcat(filename,suffix);
			ofp = fopen(strcat(filename,extent), "w");
			
			for (ijj = 1;ijj<= ttpoints;ijj++)
			{
				fprintf(ofp,"%8.2f\t", MonteTime[ijj]);
				fprintf(ofp,"%8.2f\t", avTemp[ijj]);
				fprintf(ofp,"%8.2f\t", lowEnvelope[ijj]);
				fprintf(ofp,"%8.2f\t", highEnvelope[ijj]);
				for (imonte = 1; imonte <= poolsize - 1; imonte++)
				{

					fprintf(ofp,"%6.2f\t", poolTemp[imonte][ijj]);	
				}
				fprintf(ofp,"%6.2f\n", poolTemp[poolsize][ijj]);		
			}
			fclose(ofp);

		// write out monte-carlo spectra in age spectrum format, tab-delimited

			strcpy(filename,"MONTEage_");
			strcat(filename,suffix);
			ofp = fopen(strcat(filename,extent), "w");
			
			fprintf(ofp,"%6.2f\t", bulkloss39[1] * 0.0);
			fprintf(ofp,"%6.2f\t", goal_age[1]);  // goal_age[]
			fprintf(ofp,"%6.2f\t", avAge[1]);  // average_age[]
			for (imonte = 1; imonte <= poolsize - 1; imonte++)
			{	
					fprintf(ofp,"%6.2f\t", poolAge[imonte][1]);
			}
			fprintf(ofp,"%6.2f\n", poolAge[poolsize][1]);	
				
			for (ijj = 1; ijj <= labsteps - 1; ijj++)
			{
				fprintf(ofp,"%6.2f\t", bulkloss39[ijj] * 100.0);
				fprintf(ofp,"%6.2f\t", goal_age[ijj]);
				fprintf(ofp,"%6.2f\t", avAge[ijj]);	
				for (imonte = 1; imonte <= poolsize - 1; imonte++)
				{	
					fprintf(ofp,"%6.2f\t", poolAge[imonte][ijj]);
				}
				fprintf(ofp,"%6.2f\n", poolAge[poolsize][ijj]);
				
				fprintf(ofp,"%6.2f\t", bulkloss39[ijj] * 100.0);
				fprintf(ofp,"%6.2f\t", goal_age[ijj+1]);
				fprintf(ofp,"%6.2f\t", avAge[ijj+1]);
				for (imonte = 1; imonte <= poolsize - 1; imonte++)
				{	
					fprintf(ofp,"%6.2f\t", poolAge[imonte][ijj+1]);
				}
				fprintf(ofp,"%6.2f\n", poolAge[poolsize][ijj+1]);
			}
			fprintf(ofp,"%6.2f\t", bulkloss39[labsteps] * 100.0);
			fprintf(ofp,"%6.2f\t", goal_age[labsteps]);
			fprintf(ofp,"%6.2f\t", avAge[labsteps]);
			for (imonte = 1; imonte <= poolsize - 1; imonte++)
			{	
				fprintf(ofp,"%6.2f\t", poolAge[imonte][labsteps]);
			}
			fprintf(ofp,"%6.2f\n", poolAge[poolsize][labsteps]);	
			
			fclose(ofp);
			
		// write out monte-carlo Helium ages
			if (useHelium == 1)
			{
				strcpy(filename,"MONTE_heliums_");
				strcat(filename,suffix);
				ofp = fopen(strcat(filename,extent), "w");
				fprintf(ofp,"%7.2f\n", avHeage);
				for (imonte = 1; imonte <= poolsize - 1; imonte++)
				{	
					fprintf(ofp,"%7.2f\n", Hepool[imonte]);
				}
				fprintf(ofp,"%7.2f", Hepool[poolsize]);
				fclose(ofp);
			}		
		}
		else  // we're restarting from a pre-existing pool
		{

			ifp = fopen("CRScount.in", "r");
			fscanf(ifp, "%ld", &totCRS);   // number of previous CRS iterations
			fclose(ifp);
			
   		printf("Restarting using previous pool");

   		printf("...\n");
   		
   		// first get saved tT data into pool array
   		
			ofp = fopen("CRSrestart", "r");
			for (ijj = 1;ijj<= ttpoints;ijj++)
			{
				fscanf(ofp,"%lf", &timein[ijj]);
				for (imonte = 1; imonte <= poolsize; imonte++)
				{
					fscanf(ofp,"%lf", &poolTemp[imonte][ijj]);
				}
			}	
		
				
		// now determine ages for pool

			for (imonte = 1;imonte<= poolsize;imonte++)
			{
				for (ijj = 1;ijj <= ttpoints;ijj++)
				{
					temperaturein[ijj] = poolTemp[imonte][ijj];
				}
				lovera();
				if (useHelium == 1)
				{
					helium();
					Hepool[imonte] = Heage;
				}

				for (ijj = 1; ijj <= labsteps; ijj++)
				{
					poolAge[imonte][ijj] = age[ijj];
				}
			}
			fclose(ofp);
			
  // Change directory in preparation for writing intermediate results files

	// PORT-ISSUE - make sure these standard calls work for your system
			j = 0;	
			do
			{
				if (j == 0)
				{
					sprintf(newFolder,"RESULTS-%s", suffix);		
				}
				else     // don't overwrite older folder, so rename with digit
				{
					sprintf(newFolder,"RESULTS-%s %ld", suffix, j);
				}
				j++;
			} while (mkdir(newFolder, READ_OR_WRITE ) == -1); 

		/* change to new folder */

			if( chdir( newFolder) )
			{
					printf("\nCannot change to new folder\n");
					exit(EXIT_FAILURE); 
			}
		}
		
// write out goal spectrum in age spectrum format, tab-delimited	

	ofp = fopen("goalspec.xls", "w");
	
	fprintf(ofp,"%6.2f\t", goal_loss[1] * 0.0);
	fprintf(ofp,"%6.2f\n", goal_age[1]);	
		
	for (ijj = 1; ijj <= goalsteps - 1; ijj++)
	{
		fprintf(ofp,"%6.2f\t", goal_loss[ijj] * 100.0);
		fprintf(ofp,"%6.2f\n", goal_age[ijj]);
		
		fprintf(ofp,"%6.2f\t", goal_loss[ijj] * 100.0);
		fprintf(ofp,"%6.2f\n", goal_age[ijj+1]);
	}
	
	fprintf(ofp,"%6.2f\t", goal_loss[goalsteps] * 100.0);
	fprintf(ofp,"%6.2f\n", goal_age[goalsteps]);	
	
	fclose(ofp);
/*
	ofp = fopen("loss.out", "w");
	for (ijj = 1; ijj <= labsteps - 1; ijj++)
	{
		fprintf(ofp,"%9.5f\n", bulkloss39[ijj]);
	}
	fclose(ofp);
*/

// Now load monte-carlo fits into ModelFit[]
	
	for (imonte = 1; imonte <= poolsize; imonte++)
	{
		ModelFit[imonte] = fit(imonte, poolAge);
	}
	worstfit();
	bestfit();
	
	printf("...done with %ld monte-carlo histories.\n",poolsize);
	printf("\n");
	printf("Worst fit is %7.1f (history %ld)\n",ModelFit[WorstOne],WorstOne);
	printf(" Best fit is %7.1f (history %ld)\n",ModelFit[BestOne],BestOne);
	printf("\n");
	printf("Starting to make %ld CRS histories...\n", CRSiterations);
	duration = time(NULL); // PORT-ISSUE can be deleted as timing info is not essential
	
// MAIN CRS LOOP FOLLOWS

	iCRS = 0;
	while ((iCRS < CRSiterations) && (ModelFit[WorstOne] > fitvalue)) 
	{
		iCRS++;
		    
		CRSattempts = 0;
		goodHistory = -1;
		while ((CRSattempts < 1000) && (goodHistory < 0))
		{
			CRSattempts++;
			selectsubset();
			newhistoryCRS(poolTemp);
		}
		if (CRSattempts > 1000)
		{
			printf("******************************************************************\n");
			printf("* Disaster: no good CRS history after 1000 tries. Model aborted. *\n");
			printf("******************************************************************\n");			
		}
		assert(CRSattempts <= 1000);
		
		lovera();
		if (useHelium == 1)
		{
			helium();
			Hepool[poolsize + 1] = Heage;
		}
		
		for (ijj = 1; ijj <= labsteps; ijj++)  // put new ages into pool array's last member
		{
			poolAge[poolsize + 1][ijj] = age[ijj];
		}

		for  (ijj = 1; ijj <= ttpoints; ijj++) // load new history into pool array's last element
		{
			poolTemp[poolsize + 1][ijj] = temperaturein[ijj];
		}
		
		ModelFit[poolsize + 1] = fit(poolsize + 1, poolAge);  // get fit for new age spectrum and He age
		
		if (ModelFit[poolsize + 1] < ModelFit[WorstOne])  // if a better fit...
		{
//			printf("made a swap at iteration %d  new %f7.2  old %7.2f\n",iCRS,ModelFit[poolsize + 1],ModelFit[WorstOne]);
			for (i = 1; i <= ttpoints; i++)    // swap the temperature history into pool, and...
			{
				poolTemp[WorstOne][i] = poolTemp[poolsize + 1][i];				
			}
			for (i = 1; i <= labsteps; i++)    // swap the age spectrum into pool
			{
				poolAge[WorstOne][i] = poolAge[poolsize + 1][i];
			}
			if (useHelium == 1)
			{
				Hepool[WorstOne] = Hepool[poolsize + 1];
			}
			
			ModelFit[WorstOne] = ModelFit[poolsize + 1];
			worstfit(); // if we replaced the previous worst fit, we have to find the new worst fit }
			bestfit();
		}

		if (iCRS % 50 == 0)
		{
			printf(" ...processed %4ld histories: best fit is %7.2f, worst fit is %7.2f\n",iCRS+totCRS,ModelFit[BestOne],ModelFit[WorstOne]);
		}

		// write out interim CRS time-temperature data, in tab-delimited format
		if ((iCRS == 100) || (iCRS == 200) || (iCRS == 500) || (iCRS == 1000) || (iCRS == 2000) &&  (fullreport == 1))
		{
			sort(poolAge, poolTemp, poolTime);
			averages(poolAge, poolTemp);
			nowCRS = iCRS + totCRS;
			sprintf(isuffix,"%ld",nowCRS);
			strcpy(filename,"CRStT_");
			strcat(filename,isuffix);
			strcat(filename,extent);
			ofp = fopen(filename, "w");
			for (ijj = 1;ijj<= ttpoints;ijj++)
			{
				fprintf(ofp,"%8.2f\t", timein[ijj]);
				fprintf(ofp,"%8.2f\t", avTemp[ijj]);
				fprintf(ofp,"%8.2f\t", lowEnvelope[ijj]);
				fprintf(ofp,"%8.2f\t", highEnvelope[ijj]);
				for (imonte = 1; imonte <= poolsize - 1; imonte++)
				{
					fprintf(ofp,"%6.2f\t", poolTemp[imonte][ijj]);	
				}
				fprintf(ofp,"%6.2f\n", poolTemp[poolsize][ijj]);		
			}
			fclose(ofp);
		}
			// write out rolling-progress CRS time-temperature data, in tab-delimited format
		if (iCRS % 1000 == 0)
		{
			sort(poolAge, poolTemp, poolTime);
			averages(poolAge, poolTemp);
			strcpy(filename,"CRStT_rolling.xls");
			ofp = fopen(filename, "w");
			for (ijj = 1;ijj<= ttpoints;ijj++)
			{
				fprintf(ofp,"%8.2f\t", timein[ijj]);
				fprintf(ofp,"%8.2f\t", avTemp[ijj]);
				fprintf(ofp,"%8.2f\t", lowEnvelope[ijj]);
				fprintf(ofp,"%8.2f\t", highEnvelope[ijj]);
				for (imonte = 1; imonte <= poolsize - 1; imonte++)
				{
					fprintf(ofp,"%6.2f\t", poolTemp[imonte][ijj]);	
				}
				fprintf(ofp,"%6.2f\n", poolTemp[poolsize][ijj]);		
			}
			fclose(ofp);		
			
		// write out CRS spectra in age spectrum format, tab-delimited

			strcpy(filename,"CRSage_");
			strcat(filename,isuffix);
			ofp = fopen(strcat(filename,extent), "w");
			
			fprintf(ofp,"%6.2f\t", bulkloss39[1] * 0.0);
			fprintf(ofp,"%6.2f\t", goal_age[1]);
			fprintf(ofp,"%6.2f\t", avAge[1]);			
			for (imonte = 1; imonte <= poolsize - 1; imonte++)
			{	
					fprintf(ofp,"%6.2f\t", poolAge[imonte][1]);
			}
			fprintf(ofp,"%6.2f\n", poolAge[poolsize][1]);	
				
			for (ijj = 1; ijj <= labsteps - 1; ijj++)
			{
				fprintf(ofp,"%6.2f\t", bulkloss39[ijj] * 100.0);
				fprintf(ofp,"%6.2f\t", goal_age[ijj]);
				fprintf(ofp,"%6.2f\t", avAge[ijj]);
				for (imonte = 1; imonte <= poolsize - 1; imonte++)
				{	
					fprintf(ofp,"%6.2f\t", poolAge[imonte][ijj]);
				}
				fprintf(ofp,"%6.2f\n", poolAge[poolsize][ijj]);
				
				fprintf(ofp,"%6.2f\t", bulkloss39[ijj] * 100.0);
				fprintf(ofp,"%6.2f\t", goal_age[ijj+1]);
				fprintf(ofp,"%6.2f\t", avAge[ijj+1]);
				for (imonte = 1; imonte <= poolsize - 1; imonte++)
				{	
					fprintf(ofp,"%6.2f\t", poolAge[imonte][ijj+1]);
				}
				fprintf(ofp,"%6.2f\n", poolAge[poolsize][ijj+1]);
			}
			fprintf(ofp,"%6.2f\t", bulkloss39[labsteps] * 100.0);
			fprintf(ofp,"%6.2f\t", goal_age[labsteps]);
			fprintf(ofp,"%6.2f\t", avAge[labsteps]);
			for (imonte = 1; imonte <= poolsize - 1; imonte++)
			{	
				fprintf(ofp,"%6.2f\t", poolAge[imonte][labsteps]);
			}
			fprintf(ofp,"%6.2f\n", poolAge[poolsize][labsteps]);	
			
			fclose(ofp);	
			if (useHelium == 1)
			{			
				strcpy(filename,"CRS_heliums_");
				strcat(filename,isuffix);
				ofp = fopen(strcat(filename,extent), "w");
				fprintf(ofp,"%7.2f\n", avHeage);
				for (imonte = 1; imonte <= poolsize - 1; imonte++)
				{	
					fprintf(ofp,"%7.2f\n", Hepool[imonte]);
				}
				fprintf(ofp,"%7.2f", Hepool[poolsize]);
				fclose(ofp);
			}
		}
	}  // end of main CRS loop
	
	// get final time info

	elapsed = (time(NULL) - duration)/60.0; // PORT-ISSUE can be deleted as timing not essential
	runrate = elapsed/iCRS*100; // PORT-ISSUE
	totCRS = totCRS + iCRS; // PORT-ISSUE
	
// Begin final writing of output files and summaries

	sort(poolAge, poolTemp, poolTime);
	averages(poolAge, poolTemp);

// write out run summary

	sprintf(filename,"ModelInfo_%s.txt",suffix);
	ofp = fopen(filename, "w");
	fprintf(ofp,"Model run %s\n",ctime(&now));
	fprintf(ofp,"\n");
	fprintf(ofp,"             SAMPLE INFO:   %s\n", info);
	fprintf(ofp,"             FILE SUFFIX:   %s\n", suffix);
	fprintf(ofp,"\n");
	fprintf(ofp,"          CRS ITERATIONS:  %ld\n", CRSiterations);
	fprintf(ofp,"   MODEL DURATION (m.y.):   %6.1f\n", modelduration);
	fprintf(ofp,"              TIME NODES:    %ld\n", ttpoints);
	fprintf(ofp,"CONSTRAINING BRACKETS tT:     %ld\n", nconstraints);
	fprintf(ofp,"   TIME    TMIN   TMAX\n");
	for (j = 1; j <= nconstraints; j++)
	{
		fprintf(ofp," %6.1f   %5.1f   %5.1f\n", constraintTime[j], constraintTempmin[j], constraintTempmax[j]);
	}
	fprintf(ofp,"MAX MONTE-CARLO HEATING RATE: %5.1f   MAX MONTE-CARLO COOLING RATE: %5.1f\n", monteHeatRate,monteCoolRate);
	fprintf(ofp,"        MAX CRS HEATING RATE: %5.1f           MAX CRS COOLING RATE: %5.1f\n", heatRate,coolRate);
	fprintf(ofp,"    CRS AMPLIFICATION FACTOR:   %4.2f\n", amplify);
	fprintf(ofp,"      SUBSET SIZE, POOL SIZE:   %ld   %ld\n", subsetsize, poolsize);
	fprintf(ofp,"           FITTING CRITERION:  %5.2f\n", fitvalue);
	fprintf(ofp,"              FITTING OPTION:   %ld  ", fitchoice);
	switch (fitchoice)
	{
		case 0:
			fprintf(ofp,"(step-weighted mean percent deviation)\n");
			break;
		case 1:
			fprintf(ofp,"(mean percent)\n");
			break;
		case 2:
			fprintf(ofp,"(mswd)\n");
	}

	fprintf(ofp,"          DIFFUSION GEOMETRY:   %ld  ", geometry);
	if (geometry == 1)
	{
		fprintf(ofp,"(spherical)\n");
	}
	else
	{
		fprintf(ofp,"(infinite-slab)\n");
	}
	fprintf(ofp,"              RESTART OPTION:   %ld  ", restart);
	if (restart == 0)
	{
		fprintf(ofp,"(new start from Monte-Carlo histories)\n");
	}
	else
	{
		fprintf(ofp,"(restart using values saved in file <CRSrestart>)\n");
	}
	fprintf(ofp,"DISCRETIZATION DELTA-TEMPERATURE: %5.1f\n", dt);
	fprintf(ofp,"           LOVERA SERIES CUT-OFF:   %5.1e\n", ichange);
	fprintf(ofp,"      FLAG TO WRITE FULL REPORTS:   %ld\n", fullreport);

// send helium information 
	fprintf(ofp,"\n");
	fprintf(ofp,"           ************* Helium Info *************\n");
	if (mineral == 1)
	{
		fprintf(ofp,"                      Mineral:    1 - zircon\n");
	}
	else
	{
		fprintf(ofp,"                      Mineral:    0 - apatite\n");
	}

	fprintf(ofp,"            ACTIVATION ENERGY:   %5.2f kcal/mol\n", Heact);
	fprintf(ofp,"             EFFECTIVE RADIUS: %7.2f microns     DIFZERO:  %9.3e cm2/sec\n", Heradius*1e4, HeDifzero);
	fprintf(ofp,"                  GOAL He AGE:  %6.2f Ma     ERROR IN AGE: %5.2f Ma\n", Hegoal, Hegoalerr);

	fprintf(ofp,"  WEIGHTING FACTOR FOR He AGE:  %6.2f\n", Heweight);

	if (useHelium == 1)
	{
		fprintf(ofp,"HELIUM AGE USED AS CONSTRAINT:    1 - yes\n");
	}
	else
	{
		fprintf(ofp,"HELIUM AGE USED AS CONSTRAINT:    0 - no\n");
	}
	fprintf(ofp,"\n");
	fprintf(ofp,"DOMAIN INFO\n");
	fprintf(ofp,"-----------\n");	
	fprintf(ofp,"Domains: %2ld\n", ndomains);
	assert(ndomains > 0 && ndomains < 11);
	fprintf(ofp,"      E        D0       frac.\n");
	for (j = 1; j <= ndomains; j++)
	{
		fprintf(ofp,"%2ld  %5.2f  %9.5e  %5.3f\n", j, E[j], D0[j], fraction[j]);	
	}
	fprintf(ofp,"\n");
	fprintf(ofp,"Goal Age Spectrum\n");
	fprintf(ofp,"-----------------\n");	
  fprintf(ofp,"Goal spectrum steps: %3ld\n", goalsteps);
	fprintf(ofp,"      f39     age    error  skip?\n");
	for (j = 1; j <= goalsteps; j++)
	{
		fprintf(ofp,"%3ld  %5.3f  %6.1f  %5.1f   %1ld\n", j, goal_loss[j], goal_age[j], goal_error[j], goal_skip[j]);
	}
	fprintf(ofp,"\n");
	fprintf(ofp,"Heating Schedule Actually Used\n");
	fprintf(ofp,"------------------------------\n");	
	fprintf(ofp,"Heating Steps: %3ld\n",labsteps);
	fprintf(ofp,"     Temp. (C)     Time\n");
	for (i = 1; i <= labsteps; i++)
	{
		fprintf(ofp,"%3ld    %5.1f       10.0\n", i, temperaturelab[i] - 273.15);
	}
	fclose(ofp);


// write out final CRS time-temperature data, in tab-delimited format	

	strcpy(finalfilename,"CRStT_final_");
	strcat(finalfilename,suffix);
	ofp = fopen(strcat(finalfilename,extent), "w");
	for (ijj = 1;ijj<= ttpoints;ijj++)
	{
		fprintf(ofp,"%8.2f\t", timein[ijj]);
		fprintf(ofp,"%8.2f\t", avTemp[ijj]);
		fprintf(ofp,"%8.2f\t", lowEnvelope[ijj]);
		fprintf(ofp,"%8.2f\t", highEnvelope[ijj]);
		for (imonte = 1; imonte <= poolsize - 1; imonte++)
		{
			fprintf(ofp,"%6.2f\t", poolTemp[imonte][ijj]);	
		}
		fprintf(ofp,"%6.2f\n", poolTemp[poolsize][ijj]);		
	}
	fclose(ofp);

// write out final CRS spectra in age spectrum format, tab-delimited

	strcpy(finalfilename,"CRSage_final_");
	strcat(finalfilename,suffix);
	ofp = fopen(strcat(finalfilename,extent), "w");
	
	fprintf(ofp,"%6.2f\t", bulkloss39[1] * 0.0);
	fprintf(ofp,"%6.2f\t", goal_age[1]);
	fprintf(ofp,"%6.2f\t", avAge[1]);
	for (imonte = 1; imonte <= poolsize - 1; imonte++)
	{	
			fprintf(ofp,"%6.2f\t", poolAge[imonte][1]);
	}
	fprintf(ofp,"%6.2f\n", poolAge[poolsize][1]);	
		
	for (ijj = 1; ijj <= labsteps - 1; ijj++)
	{
		fprintf(ofp,"%6.2f\t", bulkloss39[ijj] * 100.0);
		fprintf(ofp,"%6.2f\t", goal_age[ijj]);
		fprintf(ofp,"%6.2f\t", avAge[ijj]);
		for (imonte = 1; imonte <= poolsize - 1; imonte++)
		{	
			fprintf(ofp,"%6.2f\t", poolAge[imonte][ijj]);
		}
		fprintf(ofp,"%6.2f\n", poolAge[poolsize][ijj]);
		
		fprintf(ofp,"%6.2f\t", bulkloss39[ijj] * 100.0);
		fprintf(ofp,"%6.2f\t", goal_age[ijj+1]);
		fprintf(ofp,"%6.2f\t", avAge[ijj+1]);
		for (imonte = 1; imonte <= poolsize - 1; imonte++)
		{	
			fprintf(ofp,"%6.2f\t", poolAge[imonte][ijj+1]);
		}
		fprintf(ofp,"%6.2f\n", poolAge[poolsize][ijj+1]);
	}
	fprintf(ofp,"%6.2f\t", bulkloss39[labsteps] * 100.0);
	fprintf(ofp,"%6.2f\t", goal_age[labsteps]);
	fprintf(ofp,"%6.2f\t", avAge[labsteps]);
	for (imonte = 1; imonte <= poolsize - 1; imonte++)
	{	
		fprintf(ofp,"%6.2f\t", poolAge[imonte][labsteps]);
	}
	fprintf(ofp,"%6.2f\n", poolAge[poolsize][labsteps]);	
	
	fclose(ofp);
	
// write out final pool of Helium ages
	if (useHelium == 1)
	{
		strcpy(filename,"CRS_Heliums_final_");
		strcat(filename,suffix);
		ofp = fopen(strcat(filename,extent), "w");
		fprintf(ofp,"%7.2f\n", avHeage);
		for (imonte = 1; imonte <= poolsize - 1; imonte++)
		{	
			fprintf(ofp,"%7.2f\n", Hepool[imonte]);
		}
		fprintf(ofp,"%7.2f", Hepool[poolsize]);
		fclose(ofp);
	}	

// write out Modelfits if user asked for full report

	if (fullreport == 1)
	{
		strcpy(finalfilename,"Fitsfinal_");
		strcat(finalfilename,suffix);
		ofp = fopen(strcat(finalfilename,extent), "w");
		
		for (imonte = 1; imonte <= poolsize - 1; imonte++)
		{
			fprintf(ofp,"%7.3f\n", ModelFit[imonte]);	
		}
		fprintf(ofp,"%7.3f", ModelFit[poolsize]);		
	}
	
	printf("\n");
	printf("********* Arvert 4.0.0 finished with no worries! **********\n");
	printf("\n");
	if (restart == 0)
	{
		printf("    Processed %ld CRS histories\n",iCRS);
	}
	else
	{
		printf("    Processed %ld additional CRS histories\n",iCRS);
	}
	printf("    in %7.2f minutes at rate of %6.3f minutes per 100 histories\n",elapsed,runrate);//  // PORT-ISSUE

	sprintf(filename,"ModelInfo_%s.txt",suffix);  // add final info to run-summary file	
	ofp = fopen(filename, "a");
	fprintf(ofp,"\n");
	fprintf(ofp,"********* Arvert 4.0.0 finished with no worries! **********\n");
	fprintf(ofp,"\n");
	if (restart == 0)
	{
		fprintf(ofp,"Processed %ld CRS histories\n",iCRS);
	}
	else
	{
		fprintf(ofp,"Processed %ld additional CRS histories for a total of %ld\n",iCRS,totCRS);
	}

	fprintf(ofp,"in %7.2f minutes at a rate of %6.3f minutes per 100 histories\n\n",elapsed,runrate);//  // PORT-ISSUE
	
	fprintf(ofp," Best fit is %6.2f, worst fit is %6.2f\n",ModelFit[BestOne],ModelFit[WorstOne]);
	
	fclose(ofp);
	
// Finally, write out utility files for Arvert, into input directory

// Change directory back to input directory

	if( chdir( inFolder) )
	{
			printf("\nCannot change to new folder\n");
			exit(EXIT_FAILURE); 
	}

// set or update iteration counter

	ifp = fopen("CRScount.in", "w");
	fprintf(ifp, "%ld", totCRS);   // update counter file for new total iterations
	fclose(ifp);

// write out final CRS time-temperature data to CRSrestart, in tab-delimited format,
	
	ofp = fopen("CRSrestart", "w");
	for (ijj = 1;ijj<= ttpoints;ijj++)
	{
		fprintf(ofp,"%8.2f\t", timein[ijj]);
		for (imonte = 1; imonte <= poolsize - 1; imonte++)
		{
			fprintf(ofp,"%6.2f\t", poolTemp[imonte][ijj]);	
		}
		fprintf(ofp,"%6.2f\n", poolTemp[poolsize][ijj]);		
	}
	fclose(ofp);

//	free(poolAge);	
//	free(poolTemp); 
//	free(poolTime); 

    return 0;
}
