/* 	

		Function monteTt() of Arvert 4.x
		P. Zeitler, Lehigh University
		3/30/03 (update)
		
		Generate a random time-temperature history that obeys heating/cooling contraints (explicit temperature limits,
		implicit heating/cooling rate). Get time nodes spread stragetically near min and max of age spectrum
		(See Initialize() routine).
		
		Coded to allow as few as 2 nodes or as many as 10; two will always belong to 0.0 and to model duration.
		Generally recommend using at least 8 nodes to allow problem space to be adequately sampled--otherwise
		location of nodes will tend to pin solution, and tend to lead to inaccurate overprecise overconvergence.
  	
*/

#include "arvert.h"

void monteTt(void)
{
	extern long RanSeedZero;
	extern long ttpoints;
	extern double MonteTime[nodes], MonteTemp[nodes];
	extern double temperaturein[nodes],timein[nodes];
	extern double lowerlimit[nodes], upperlimit[nodes];
//	extern double modelduration;
	extern double monteHeatRate,monteCoolRate;
//	extern double Time_at_node[nodes];

	long start, index;
	double highlimit,lowlimit, ratelimit;

// NOTE: MonteTime [] stores time coords of Monte Carlo nodes in problem frame of reference.

// Determined time nodes from age spectrum in Initialize.c

// Try making a thermal history. To avoid bias, start at a randomly selected node and work both directions.
	
// get a starting point
	start = 1 + floor(ran3(RanSeedZero) * ttpoints);
	if (start == ttpoints + 1) //  very unlikely case that random exactly = 1
	{
		start = ttpoints;
	}	
		
// get temperature for starting point
	MonteTemp[start] = lowerlimit[start] + ran3(RanSeedZero) * (upperlimit[start] - lowerlimit[start]);
//			printf("%d  %6.1f  %6.1f  %6.1f  %6.1f\n",time_node[start],MonteTime[start],MonteTemp[start],lowerlimit[start],upperlimit[start]);			

// generate temperatures at times older than starting point
	if (start != 1)
	{
		for (index = start - 1; index >= 1; index--)  //working backward, implict cooling limit determines implicit max limit, and vice versa
			{
				ratelimit = MonteTemp[index+1] + monteCoolRate*(MonteTime[index] - MonteTime[index+1]);  //first, use implicit cooling limit
				if (ratelimit < upperlimit[index])
				{
					highlimit = ratelimit;
				}
				else
				{
					highlimit = upperlimit[index];
				}
			
				ratelimit = MonteTemp[index+1] - monteHeatRate*(MonteTime[index] - MonteTime[index+1]);  //next, use implicit heating limit
				if (ratelimit > lowerlimit[index])
				{
					lowlimit = ratelimit;
				}
				else
				{
					lowlimit = lowerlimit[index];
				}
				
				MonteTemp[index] = lowlimit + ran3(RanSeedZero) * (highlimit - lowlimit);
//			printf("%d  %6.1f  %6.1f  %6.1f  %6.1f\n",index,MonteTime[index],MonteTemp[index],lowlimit,highlimit);			

			}
  }

// generate temperatures at times younger than starting point
	if (start != ttpoints)
	{
		for (index = start + 1; index <= ttpoints; index++)
		{

				ratelimit = MonteTemp[index-1] - monteCoolRate*(MonteTime[index-1] - MonteTime[index]);  //first, use implicit cooling limit
				if (ratelimit > lowerlimit[index])
				{
					lowlimit = ratelimit;
				}
				else
				{
					lowlimit = lowerlimit[index];
				}
			
				ratelimit = MonteTemp[index-1] + monteHeatRate*(MonteTime[index-1] - MonteTime[index]);  //next, use implicit heating limit
				if (ratelimit < upperlimit[index])
				{
					highlimit = ratelimit;
				}
				else
				{
					highlimit = upperlimit[index];
				}


			MonteTemp[index] = lowlimit + ran3(RanSeedZero) * (highlimit - lowlimit);
//			printf("%d  %6.1f  %6.1f  %6.1f  %6.1f\n",index,MonteTime[index],MonteTemp[index],lowlimit,highlimit);			
		}
	}

/*	
	for (index = 1; index <= ttpoints; index++)
	{
		printf("%d  %6.1f  %6.1f\n",index,MonteTime[index],MonteTemp[index]);
	}
*/

	//need to interface with inputs for Lovera

	for (index = 1; index <= ttpoints; index++)
	{
		timein[index] = MonteTime[index];
		temperaturein[index] = MonteTemp[index];
	}

/*
	printf("***************************\n");
	printf("node  timein   tempin\n");	
	for (i=1; i<=ttpoints; i++)
	{
			printf("%d  %6.1f  %6.1f\n",i,timein[i],temperaturein[i]);
	}
	printf("***************************\n");
		

	printf("PAUSED for input\n");
	printf("  QUIT to abort model  --> ");
	scanf("%d", &i);
	*/
	
	return;
}